'' ==========================================================
''
''   File....... jm_nec6121_rx_demo_JT.spin
''   Purpose.... NEC6121 IR protocol receiver demo
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''   E-mail..... jon@jonmcphalen.com
''   Updated.... 27 FEB 2010
''
''  Modified, expanded, and comments added by Jon Titus
''  December 2, 2013.
''
'' ==========================================================
{{
''***********************************************
''*  Program version EX18.2.spin
''*  Author: Jon Titus 12-02-2013 Rev. 2
''*  Titus Revisions Copyright 2013
''*  Program used to obtain a 32-bit packet in NEC
''*  infrared (IR) remote-control format. This program
''*  extracts the command and address information, checks
''*  for bit errors and displays the associated pressed
''*  key information.
''*  Uses P8X32A Propeller-I board.
''*  
''*  SET BAUD RATE AT 115,200 IN PST!!!
''***********************************************
}}


'Set up clock frequency and calculate number of clock cycles in a millisecond.
con

  _clkmode = xtal1 + pll16x                             'Set clock mode for external slow-speed crystal
                                                        'and internal PLL multiplier of 16
  _xinfreq = 5_000_000                                  'External crystal at 5 MHz, so clock frequency
                                                        'equals 5 MHz * 16 = 80 MHz.
  CLK_FREQ = ((pll16x) >> 6) * _xinfreq                 'Get pll16x constant: 100_0000_0000               
                                                        'and shift bits to get: 1_0000, or 16.
                                                        'Multiply by clock frequency to get 80,000,000 Hz
  MS_001   = CLK_FREQ / 1_000                           'Divide by 1000 to get clock "ticks" for a one-
                                                        'millisecond period (80,000)

'Set up data for remote-control unit in use (here Sparkfun 9-button IR
'remote control). Use only the command byte here, not entire 32-bit
'data packet to identify pressed key.
con
  
  KEY_ON_OFF  = $1B
  KEY_A       = $1F
  KEY_B       = $1E
  KEY_C       = $1A  
  KEY_UP      = $05
  KEY_DOWN    = $00
  KEY_RIGHT   = $01
  KEY_LEFT    = $08
  KEY_CENTER  = $04

'PST Control codes
  CR          = 13          'ASCII carriage-return code for a new line in PST

VAR
    byte address            'storage for address 
    byte address_comp       'storage for address-complement
    byte address_check      'storage for result of address bit-error check
    byte command            'storage for command
    byte command_comp       'storage for command-complement
    byte command_check      'storage for result of command bit-error check
    
obj

  ir   : "jm_nec6121_rx"                                'Use this object to decode NEC packet 
  term : "fullduplexserial"                             'Object to control Parallax Serial
                                                        'Terminal (PST) to display results
pub main | code

  ir.init(8)                                            'IR-sensor input on Propeller I/O P8
  
  term.start(31, 30, %0000, 115_200)                    'Start PST with these settings
                                                        'set Baud Rate in PST to 115,200 
  pause(1)                                              'Wait here for 1 millisecond

  repeat                                                'Run repeat loop forever

    code := ir.rx                                       'Get received address and command from the
                                                        'jm_nec6121_rx.spin object
    address := code & $FF                               'Extract address                                  
    address_comp := (code >> 8) & $FF                   'Extract address complement
    command := (code >> 16) & $FF                       'Extract command 
    command_comp := (code >> 24) & $FF                  'Extract command complement

    'Print legend for corresponding received command                                                            
    case command                                          'Code translation for Sparkfun remote control
      KEY_ON_OFF  : term.str(string("On/Off......... "))  'These translations use only the command byte
      KEY_A       : term.str(string("A.............. "))  'and not the entire 32-bit data packet.
      KEY_B       : term.str(string("B.............. "))
      KEY_C       : term.str(string("C.............. "))
      KEY_UP      : term.str(string("Up............. "))
      KEY_DOWN    : term.str(string("Down........... "))
      KEY_RIGHT   : term.str(string("Right.......... "))      
      KEY_LEFT    : term.str(string("Left........... "))
      KEY_CENTER  : term.str(string("Center......... "))
      other       : term.str(string("Key Error...... "))

    'Print 32-bit hex code as received               
    term.hex(code, 8)                                   'After the command, print the 8-digit hex code               
    term.tx(CR)                                         'Go to a new line for the next packet's data

    'Print hex values for command and address information
    term.str(string("Address:  "))                      'Print the address value in hex
    term.hex(address, 2)
    term.tx(CR)
    term.str(string("Address Complement  "))            'Print the address-complement in hex
    term.hex(address_comp, 2)
    term.tx(CR)
    term.str(string("Command  "))                       'Print the command in hex
    term.hex(command, 2)
    term.tx(CR)
    term.str(string("Command Complement  "))            'Print the command-complement in hex
    term.hex(command_comp, 2)
    term.tx(CR)
    term.tx(CR)

    'Check address and command information for a bit error
    address_check := 1 + address + address_comp         'Add address, address-complement, and 1
    command_check := 1 + command + command_comp         'Add command, command-complement, and 1
    if (address_check <> 0)                             'If address_check not zero, print error msg
        term.str(string("Address Error"))
    if (command_check <> 0)                             'If command_check not zero, print error msg
        term.str(string("Command Error"))    
            
    pause(250)                                          'Wait here for the release of the pressed button
                                                        'Pause of 250 milliseconds

'Millisecond timing routine
pub pause(ms) | t                                       'Local variable t

  t := cnt                                              'Get System Counter value
  repeat ms                                             'Repeat this loop for each millisecond
    waitcnt(t += MS_001)                                'Add MS_001 value to t each time through
                                                        'this loop           

 '---end of Program EX18.2-----

 {{

  Original program Copyright (c) 2010 Jon McPhalen (aka Jon Williams)
  Revisions to original program Copyright (c) 2013, Jonathan A. Titus
  
  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}  